unit DataPrn;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, DB, DBTables, Printers;

type
  TODDataPrinter = class(TComponent)
  private
    FDataSource: TDataSource;
    FCustomize: Boolean;
    FTimeStamp: Boolean;
    FPageNumbers: Boolean;
{   FPrintToFile: Boolean;}
    FPrintDialog: Boolean;
    FMargin: Smallint;
    FColumnGap: Smallint;
    FTitle: string;
    FScript: TStrings;
    FTitleFont, FScriptFont, FHeaderFont, FBodyFont: TFont;
    procedure SetDataSource(Value: TDataSource);
    procedure SetTitleFont(Value: TFont);
    procedure SetScriptFont(Value: TFont);
    procedure SetHeaderFont(Value: TFont);
    procedure SetBodyFont(Value: TFont);
  protected
    procedure DoPrint;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function Print(const Title: string; Script: TStrings): Boolean;
  published
    property DataSource: TDataSource read FDataSource write SetDataSource;
    property Customize: Boolean read FCustomize write FCustomize default False;
    property TimeStamp: Boolean read FTimeStamp write FTimeStamp default True;
    property PageNumbers: Boolean read FPageNumbers write FPageNumbers default True;
{   property PrintToFile: Boolean read FPrintToFile write FPrintToFile default True;}
    property PrintDialog: Boolean read FPrintDialog write FPrintDialog default True;
    property Margin: Smallint read FMargin write FMargin default 50;
    property ColumnGap: Smallint read FColumnGap write FColumnGap default 4;
    property TitleFont: TFont read FTitleFont write SetTitleFont;
    property ScriptFont: TFont read FScriptFont write SetScriptFont;
    property HeaderFont: TFont read FHeaderFont write SetHeaderFont;
    property BodyFont: TFont read FBodyFont write SetBodyFont;
  end;

procedure Register;

implementation

type
  TiqColWidths = array[0..37] of Integer;

procedure Register;
begin
  RegisterComponents('Orbital', [TODDataPrinter]);
end;

constructor TODDataPrinter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTitleFont := TFont.Create;
  FTitleFont.Name := 'Arial';
  FTitleFont.Size := 12;
  FTitleFont.Style := [fsBold];
  FScriptFont := TFont.Create;
  FScriptFont.Name := FTitleFont.Name;
  FScriptFont.Size := 9;
  FHeaderFont := TFont.Create;
  FHeaderFont.Name := FTitleFont.Name;
  FHeaderFont.Size := 10;
  FHeaderFont.Style := [fsUnderline];
  FBodyFont := TFont.Create;
  FBodyFont.Name := FTitleFont.Name;
  FBodyFont.Size := 10;
  FTimeStamp := True;
  FPageNumbers := True;
{ FPrintToFile := True;}
  FPrintDialog := True;
  FMargin := 50;
  FColumnGap := 4;
end;

destructor TODDataPrinter.Destroy;
begin
  FTitleFont.Free;
  FScriptFont.Free;
  FHeaderFont.Free;
  FBodyFont.Free;
  inherited Destroy;
end;

procedure TODDataPrinter.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FDataSource) then
    FDataSource := nil
end;

function TODDataPrinter.Print(const Title: string; Script: TStrings): Boolean;
begin
  if (FDataSource = nil) or (FDataSource.DataSet = nil) or
     (FDataSource.DataSet.Active = False) then
    raise EComponentError.Create('Inaccessable dataset');
  FTitle := Title;
  FScript := Script;
  Result := True;
  if FPrintDialog then
    with TPrintDialog.Create(Self) do
    try
{     if FPrintToFile then Options := [poPrintToFile];}
      Result := Execute;
{     FToFile := PrintToFile;}
    finally
      Free;
    end;
{ if Result and FToFile then
    with TOpenDialog.Create(Self) do
    try
      DefaultExt := 'TXT';
      Filter := 'Text file (TXT)|*.TXT';
      Options := [ofOverwritePrompt, ofHideReadOnly];
      FileName := FTitle;
      Result := Execute;
      if Result then FTitle := FileName;
    finally
      Free;
    end;}
  if Result then DoPrint;
end;

procedure TODDataPrinter.DoPrint;

  procedure PrintHeaders(var ATop: Integer; var Widths: TiqColWidths);
  var
    i, n, a: Integer;
    s: string;
  begin
    with Printer.Canvas, FDataSource.Dataset do begin
      Font.Assign(FHeaderFont);
      a := TextWidth('a');   {get average pixels per char}
      n := FMargin;
      for i := 0 to FieldCount-1 do
        with Fields[i] do
          if Visible then begin                 {get char width -}
            if (TextWidth(DisplayLabel) div a) > DisplayWidth then
              Widths[i] := TextWidth(DisplayLabel) else
              Widths[i] := DisplayWidth * a;
            if Fields[i] is TNumericField then
              s := '' else    {right align}
              s := '-';       {left align}
            TextOut(n, ATop, Format('%' + s + '*.*s',
              [Widths[i] div a, Widths[i] div a, DisplayLabel]));
            Inc(n, Widths[i] + FColumnGap);
          end;
    end;
  end;

var
  i, aTop, aLeft, aHeight, aWidth, aAve: Integer;
  s: string;
  aWidths: TiqColWidths;
  m: TBookmark;
begin
  Printer.BeginDoc;
  with Printer, Canvas do
  try
    //print title --------------------------------------------
    Font.Assign(FTitleFont);
    if FTitle <> '' then
      TextOut(FMargin, 0, FTitle);
    aTop := TextHeight('A');
    Font.Assign(FBodyFont);
    if FTimeStamp then
      TextOut(PageWidth div 2, 0, 'Printed: ' +
        FormatDateTime('dd/mm/yyyy hh:mm AM/PM', Now));
    if FPageNumbers then begin
      s := 'Page ' + IntToStr(PageNumber);
      TextOut(PageWidth - TextWidth(s) - 50, 0, s);
    end;
    //print script -------------------------------------------
    if FScript <> nil then begin
      Font.Assign(FScriptFont);
      aHeight := TextHeight('D');
      for i := 0 to FScript.Count-1 do begin
        TextOut(FMargin, aTop, FScript[i]);
        Inc(aTop, aHeight);
      end;
      Inc(aTop, aHeight div 2);
    end;
    //print headers & body ---------------------------------------
    PrintHeaders(aTop, aWidths);
    Font.Assign(FBodyFont);
    aHeight := TextHeight('B');
    Inc(aTop, aHeight + (aHeight div 3));
    aAve := TextWidth('a');
    with FDataSource.Dataset do
    begin
      m := GetBookmark;
      DisableControls;
      try
        First;
        while not EOF do begin
          if aTop + aHeight > PageHeight then begin
            NewPage;
            aTop := 0;
            if FPageNumbers then begin
              s := 'Page ' + IntToStr(PageNumber);
              TextOut(PageWidth - TextWidth(s) - 50, aTop, s);
              Inc(aTop, aHeight * 2);
            end;
            PrintHeaders(aTop, aWidths);
            Font.Assign(FBodyFont);
          end;
          aLeft := FMargin;
          for i := 0 to FieldCount-1 do
            if Fields[i].Visible then begin
              aWidth := aWidths[i] div aAve;
              if Fields[i] is TCurrencyField then
                s := Format('%*m', [aWidth, Fields[i].AsFloat])
              else if Fields[i] is TFloatField then
              begin
                s := FormatFloat(TFloatField(Fields[i]).DisplayFormat,
                      Fields[i].AsFloat);
                s := Format('%*.*s', [aWidth, aWidth, s]);
{               with TFloatField(Fields[i]) do
                  if Precision > 4 then
                    s := Format('%*.*f', [aWidth, 4, AsFloat])
                  else
                    s := Format('%*.*f', [aWidth, Precision, AsFloat])}
              end
              else if Fields[i] is TNumericField then
                s := Format('%*.*s', [aWidth, aWidth, Fields[i].AsString])
              else if Fields[i] is TDateField then
                s := FormatDateTime(TDateField(Fields[i]).DisplayFormat,
                      Fields[i].AsDateTime)
              else  {display as string}
                s := Format('%-*.*s', [aWidth, aWidth, Fields[i].AsString]);
              TextOut(aLeft, aTop, s);
{             r.Left := aLeft;
              r.Top := aTop;
              r.Bottom := aTop + aHeight;
              r.Right := aLeft + aWidths[i];
              TextRect(r, 0, 0, s);}
              Inc(aLeft, aWidths[i] + FColumnGap);
            end;
          Next;
          Inc(aTop, aHeight);
        end;
      finally
        GotoBookmark(m);
        FreeBookmark(m);
        EnableControls;
      end;
    end;
  finally
    Printer.EndDoc;
  end;
end;

procedure TODDataPrinter.SetDataSource(Value: TDataSource);
begin
  FDataSource := Value;
  if Value <> nil then
    Value.FreeNotification(Self);
end;

procedure TODDataPrinter.SetTitleFont(Value: TFont);
begin
  FTitleFont.Assign(Value);
end;

procedure TODDataPrinter.SetScriptFont(Value: TFont);
begin
  FScriptFont.Assign(Value);
end;

procedure TODDataPrinter.SetHeaderFont(Value: TFont);
begin
  FHeaderFont.Assign(Value);
end;

procedure TODDataPrinter.SetBodyFont(Value: TFont);
begin
  FBodyFont.Assign(Value);
end;

end.
